searchNodes=[{"doc":"This module is used to interact with the global state needed to correctly generate version 1 and version 6 UUIDs. This state consists of two atomic values: the last timestamp at which a UUID was generated; and the current clock sequence value. On init, the clock sequence is seeded with a random value provided by interpreting bits from the cryptographic random number generator as an unsigned 14 bit integer. On subsequent generations, the clock sequence is incremented by 1 if a UUID has already been generated with the same last timestamp. If the timestamp has changed, the clock sequence remains unchanged.","ref":"Uniq.Generator.html","title":"Uniq.Generator","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Uniq.Generator.html#child_spec/1","title":"Uniq.Generator.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"Uniq.Generator.html#init/1","title":"Uniq.Generator.init/1","type":"function"},{"doc":"Generates the next initial state for UUID creation.","ref":"Uniq.Generator.html#next/0","title":"Uniq.Generator.next/0","type":"function"},{"doc":"","ref":"Uniq.Generator.html#start_link/1","title":"Uniq.Generator.start_link/1","type":"function"},{"doc":"","ref":"Uniq.Macros.html","title":"Uniq.Macros","type":"module"},{"doc":"","ref":"Uniq.Macros.html#defextension/2","title":"Uniq.Macros.defextension/2","type":"macro"},{"doc":"","ref":"Uniq.Macros.html#defshim/3","title":"Uniq.Macros.defshim/3","type":"macro"},{"doc":"This module provides RFC 4122 compliant universally unique identifiers (UUIDs). See the README for general usage information.","ref":"Uniq.UUID.html","title":"Uniq.UUID","type":"module"},{"doc":"Compares two UUIDs, using their canonical 128-bit integer form, as described in RFC 4122. You may provide the UUIDs in either string, binary, or as a Uniq.UUID struct.","ref":"Uniq.UUID.html#compare/2","title":"Uniq.UUID.compare/2","type":"function"},{"doc":"See :binary.decode_hex/1 .","ref":"Uniq.UUID.html#decode_hex/1","title":"Uniq.UUID.decode_hex/1","type":"function"},{"doc":"See :binary.encode_hex/1 .","ref":"Uniq.UUID.html#encode_hex/1","title":"Uniq.UUID.encode_hex/1","type":"function"},{"doc":"This function parses the given UUID, in any of the supported encodings/formats, and produces the information gleaned from the encoded data. Two styles of information are supported, depending on whether the function is called via the compatibility shim for :elixir_uuid , or directly. You may pass :struct or :keyword manually if you wish to express a preference for one style or the other. The :struct form is the UUID structure used internally by this library, and it contains all of the information needed to re-encode the UUID as binary. The :keyword form matches 1:1 the keyword list produced by UUID.info/1 provided by the :elixir_uuid library, and it contains slightly less information, but is useful for compatibility with legacy code that operates on that structure. Examples iex&gt; Uniq.UUID . info ( &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot; , :keyword ) { :ok , [ uuid : &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot; , binary : &lt;&lt; 135 , 13 , 248 , 232 , 49 , 7 , 68 , 135 , 131 , 22 , 129 , 224 , 137 , 184 , 194 , 207 &gt;&gt; , type : :default , version : 4 , variant : :rfc4122 ] } iex&gt; Uniq.UUID . info ( &quot;870df8e8-3107-4487-8316-81e089b8c2cf&quot; ) { :ok , % Uniq.UUID { format : :default , version : 4 , variant : &lt;&lt; 2 :: 2 &gt;&gt; , time : 326283406408022248 , seq : 790 , node : &lt;&lt; 129 , 224 , 137 , 184 , 194 , 207 &gt;&gt; , bytes : &lt;&lt; 135 , 13 , 248 , 232 , 49 , 7 , 68 , 135 , 131 , 22 , 129 , 224 , 137 , 184 , 194 , 207 &gt;&gt; , } }","ref":"Uniq.UUID.html#info/2","title":"Uniq.UUID.info/2","type":"function"},{"doc":"Like info/1 , but raises if the input UUID is invalid.","ref":"Uniq.UUID.html#info!/2","title":"Uniq.UUID.info!/2","type":"function"},{"doc":"Parses a Elixir.Uniq.UUID from a binary. Supported formats include human-readable strings, as well as the raw binary form of the UUID. Examples iex&gt; { :ok , uuid } = Uniq.UUID . parse ( &quot;f81d4fae-7dec-11d0-a765-00a0c91e6bf6&quot; ) { :ok , % Uniq.UUID { bytes : &lt;&lt; 248 , 29 , 79 , 174 , 125 , 236 , 17 , 208 , 167 , 101 , 0 , 160 , 201 , 30 , 107 , 246 &gt;&gt; , format : :default , node : &lt;&lt; 0 , 160 , 201 , 30 , 107 , 246 &gt;&gt; , seq : 10085 , time : 130742845922168750 , variant : &lt;&lt; 2 :: size ( 2 ) &gt;&gt; , version : 1 } } ...&gt; { :ok , % Uniq.UUID { uuid | format : :urn } } == Uniq.UUID . parse ( &quot;urn:uuid:f81d4fae-7dec-11d0-a765-00a0c91e6bf6&quot; ) true iex&gt; match? ( { :ok , % Uniq.UUID { format : :default , version : 1 } } , Uniq.UUID . uuid1 ( ) |&gt; Uniq.UUID . parse ( ) ) true","ref":"Uniq.UUID.html#parse/1","title":"Uniq.UUID.parse/1","type":"function"},{"doc":"This function takes a UUID string in any of the formats supported by to_string/1 , and returns the raw, binary-encoded form.","ref":"Uniq.UUID.html#string_to_binary!/1","title":"Uniq.UUID.string_to_binary!/1","type":"function"},{"doc":"Formats a Elixir.Uniq.UUID as a string, using the format it was originally generated with. See to_string/2 if you want to specify what format to produce.","ref":"Uniq.UUID.html#to_string/1","title":"Uniq.UUID.to_string/1","type":"function"},{"doc":"Same as to_string/1 , except you can specify the desired format. The format can be one of the following: :default , produces strings like &quot;f81d4fae-7dec-11d0-a765-00a0c91e6bf6&quot; :urn , produces strings like &quot;urn:uuid:f81d4fae-7dec-11d0-a765-00a0c91e6bf6&quot; :hex , produces strings like &quot;f81d4fae7dec11d0a76500a0c91e6bf6&quot; :slug , produces strings like &quot;-B1Prn3sEdCnZQCgyR5r9g==&quot; :raw , produces the raw binary encoding of the uuid in 128 bits","ref":"Uniq.UUID.html#to_string/2","title":"Uniq.UUID.to_string/2","type":"function"},{"doc":"Generates a UUID using the version 1 scheme, as described in RFC 4122 This scheme is based on a few key properties: A timestamp, based on the count of 100-nanosecond intervals since the start of the Gregorian calendar, i.e. October 15th, 1582, in Coordinated Universal Time (UTC). A clock sequence number, used to ensure that UUIDs generated with the same timestamp are still unique, by incrementing the sequence each time a UUID is generated with the same timestamp as the last UUID that was generated. This sequence is initialized with random bytes at startup, to protect against conflicts. A node identifier, which is based on the MAC address of one of the network interfaces on the system, or if unavailable, using random bytes. In our case, we specifically look for the first network interface returned by :inet.getifaddrs/0 that is up, broadcastable, and has a hardware address, otherwise falling back to cryptographically strong random bytes.","ref":"Uniq.UUID.html#uuid1/1","title":"Uniq.UUID.uuid1/1","type":"function"},{"doc":"This function is the same as uuid/1 , except the caller provides the clock sequence value and the node identifier (which must be a 6-byte binary). See uuid/1 for details.","ref":"Uniq.UUID.html#uuid1/3","title":"Uniq.UUID.uuid1/3","type":"function"},{"doc":"Generates a UUID using the version 3 scheme, as described in RFC 4122 This scheme provides the means for generating UUIDs deterministically, given a namespace and a name. This means that with the same inputs, you get the same UUID as output. The main difference between this and the version 5 scheme, is that version 3 uses MD5 for hashing, and version 5 uses SHA1. Both hashes are deprecated these days, but you should prefer version 5 unless otherwise required. In this scheme, the timestamp, clock sequence and node value are constructed from the namespace and name, as described in RFC 4122, Section 4.3. Namespaces You may choose one of several options for namespacing your UUIDs: Use a predefined namespace. These are provided by RFC 4122 in order to provide namespacing for common types of names. See below. Use your own namespace. For this, simply generate a UUID to represent the namespace. You may provide this UUID in whatever format is supported by parse/1 . Use nil . This is bound to a special-case UUID that has no intrinsic meaning, but is valid for use as a namespace. The set of predefined namespaces consist of the following: :dns , intended for namespacing fully-qualified domain names :url , intended for namespacing URLs :oid , intended for namespacing ISO OIDs :x500 , intended for namespacing X.500 DNs (in DER or text output format) Notes One thing to be aware of with version 3 and 5 UUIDs, is that unlike version 1 and 6, the lexicographical ordering of UUIDs of generated one after the other, is entirely random, as the most significant bits are dependent upon the hash of the namespace and name, and thus not based on time or even the lexicographical ordering of the name. This is generally worth the tradeoff in favor of determinism, but it is something to be aware of. Likewise, since the generation is deterministic, care must be taken to ensure that you do not try to use the same name for two different objects within the same namespace. This should be obvious, but since the other schemes are not sensitive in this way, it is worth calling out.","ref":"Uniq.UUID.html#uuid3/3","title":"Uniq.UUID.uuid3/3","type":"function"},{"doc":"Generates a UUID using the version 4 scheme, as described in RFC 4122 This scheme is like the version 1 scheme, except it uses randomly generated data for the timestamp, clock sequence, and node fields. This scheme is the closest you can get to truly unique identifiers, as they are based on truly random (or pseudo-random) data, so the chances of generating the same UUID twice is astronomically small. Notes The version 4 scheme does have some deficiencies. Namely, since they are based on random data, the lexicographical ordering of the resulting UUID is itself random, which can play havoc with database indices should you choose to use UUIDs for primary keys. It is strongly recommended to consider the version 6 scheme instead. They are almost the same as a version 1 UUID, but with improved semantics that combine some of the beneficial traits of version 4 UUIDs without the lexicographical ordering downsides. The only caveat to that recommendation is if you need to pass them through a system that inspects the UUID encoding itself and doesn't have preliminary support for version 6.","ref":"Uniq.UUID.html#uuid4/1","title":"Uniq.UUID.uuid4/1","type":"function"},{"doc":"Generates a UUID using the version 5 scheme, as described in RFC 4122 This scheme provides the means for generating UUIDs deterministically, given a namespace and a name. This means that with the same inputs, you get the same UUID as output. The main difference between this and the version 5 scheme, is that version 3 uses MD5 for hashing, and version 5 uses SHA1. Both hashes are deprecated these days, but you should prefer version 5 unless otherwise required. In this scheme, the timestamp, clock sequence and node value are constructed from the namespace and name, as described in RFC 4122, Section 4.3. Namespaces You may choose one of several options for namespacing your UUIDs: Use a predefined namespace. These are provided by RFC 4122 in order to provide namespacing for common types of names. See below. Use your own namespace. For this, simply generate a UUID to represent the namespace. You may provide this UUID in whatever format is supported by parse/1 . Use nil . This is bound to a special-case UUID that has no intrinsic meaning, but is valid for use as a namespace. The set of predefined namespaces consist of the following: :dns , intended for namespacing fully-qualified domain names :url , intended for namespacing URLs :oid , intended for namespacing ISO OIDs :x500 , intended for namespacing X.500 DNs (in DER or text output format) Notes One thing to be aware of with version 3 and 5 UUIDs, is that unlike version 1 and 6, the lexicographical ordering of UUIDs of generated one after the other, is entirely random, as the most significant bits are dependent upon the hash of the namespace and name, and thus not based on time or even the lexicographical ordering of the name. This is generally worth the tradeoff in favor of determinism, but it is something to be aware of. Likewise, since the generation is deterministic, care must be taken to ensure that you do not try to use the same name for two different objects within the same namespace. This should be obvious, but since the other schemes are not sensitive in this way, it is worth calling out.","ref":"Uniq.UUID.html#uuid5/3","title":"Uniq.UUID.uuid5/3","type":"function"},{"doc":"Generates a UUID using the proposed version 6 scheme, found here . This is a draft extension of RFC 4122, but has not yet been formally accepted. Version 6 provides the following benefits over versions 1 and 4: Like version 1, it is time-based, but unlike version 1, it is naturally sortable by time in its raw binary encoded form Like version 4, it provides better guarantees of uniqueness and privacy, by basing itself on random or pseudo-random data, rather than MAC addresses and other potentially sensitive information. Unlike version 4, which tends to interact poorly with database indices due to being derived entirely from random or pseudo-random data; version 6 ensures that the most significant bits of the binary encoded form are a 1:1 match with the most significant bits of the timestamp on which it was derived. This guarantees that version 6 UUIDs are naturally sortable in the order in which they were generated (with some randomness among those which are generated at the same time). There have been a number of similar proposals that address the same set of flaws. For example: KSUID ULID Systems that do not involve legacy UUIDv1 SHOULD consider using UUIDv7 instead.","ref":"Uniq.UUID.html#uuid6/1","title":"Uniq.UUID.uuid6/1","type":"function"},{"doc":"Generates a UUID using the proposed version 7 scheme, found here . This is a draft extension of RFC 4122, but has not yet been formally accepted. UUID version 7 features a time-ordered value field derived from the widely implemented and well known Unix Epoch timestamp source, the number of milliseconds seconds since midnight 1 Jan 1970 UTC, leap seconds excluded. As well as improved entropy characteristics over versions 1 or 6. Implementations SHOULD utilize UUID version 7 over UUID version 1 and 6 if possible.","ref":"Uniq.UUID.html#uuid7/1","title":"Uniq.UUID.uuid7/1","type":"function"},{"doc":"Returns true if the given string is a valid UUID. Options strict: boolean , if true, requires strict RFC 4122 conformance, i.e. version 6 is considered invalid","ref":"Uniq.UUID.html#valid?/2","title":"Uniq.UUID.valid?/2","type":"function"},{"doc":"","ref":"Uniq.UUID.html#t:format/0","title":"Uniq.UUID.format/0","type":"type"},{"doc":"","ref":"Uniq.UUID.html#t:formatted/0","title":"Uniq.UUID.formatted/0","type":"type"},{"doc":"","ref":"Uniq.UUID.html#t:info/0","title":"Uniq.UUID.info/0","type":"type"},{"doc":"","ref":"Uniq.UUID.html#t:namespace/0","title":"Uniq.UUID.namespace/0","type":"type"},{"doc":"","ref":"Uniq.UUID.html#t:t/0","title":"Uniq.UUID.t/0","type":"type"},{"doc":"Uniq provides generation, formatting, parsing, and analysis of RFC 4122 UUIDs, with support for the draft UUIDv6 extension. It is a package for Elixir projects, and can be found on Hex as :uniq .","ref":"readme.html","title":"About","type":"extras"},{"doc":"Follows the RFC 4122 specification, i.e. supports UUID versions 1, 3, 4, and 5 as described in the RFC Supports UUIDv6 and UUIDv7, which are described in a proposed extension for RFC 4122, and improve upon desirable traits of both UUIDv1 and UUIDv4 to provide the best of both, while removing their downsides. See here for more information on how it does so. Supports formatting UUIDs as canonical strings (e.g. 6ba7b810-9dad-11d1-80b4-00c04fd430c8 , with or without the dashes), as URNs, e.g. urn:uuid:6ba7b810-9dad-11d1-80b4-00c04fd430c8 , as well as a compact, 22-character, base64-encoded format using a URI-safe alphabet (e.g. a6e4EJ2tEdGAtADAT9QwyA ). Case-insensitive, i.e. 6ba7b810-9dad-11d1-80b4-00c04fd430c8 and 6BA7B810-9DAD-11D1-80B4-00C04FD430C8 have the same encoding Supports Ecto out of the box, just use Uniq.UUID as the type of a field where you would use Ecto.UUID . See the docs for more info. Can be used as a drop-in replacement for elixir_uuid , see the docs for details on migrating","ref":"readme.html#features","title":"About - Features","type":"extras"},{"doc":"def deps do [ { :uniq , &quot;~&gt; 0.1&quot; } ] end","ref":"readme.html#installation","title":"About - Installation","type":"extras"},{"doc":"The primary API is provided by the Uniq.UUID module. To generate UUIDs, pick the version you want, and call the appropriate generator. For example: uuid1/0 , generates UUIDv1 and formats it as a human-readable string, i.e. 6ba7b810-9dad-11d1-80b4-00c04fd430c8 uuid1/1 , generates UUIDv1 in the specified format uuid3/2 , generates UUIDv3 using the provided namespace and name, and formats it as a human-readable string uuid3/3 , generates UUIDv3 using the provided namespace and name, in the specified format See the docs for the full set of functions available. You can also convert UUID strings to/from the human-readable and binary formats; parse UUID strings/binaries; and determine their validity.","ref":"readme.html#usage","title":"About - Usage","type":"extras"},{"doc":"You can use the type provided by this library in lieu of Ecto.UUID , simply use Uniq.UUID where you would use Ecto.UUID , and specify :binary or :string as the type of the column in your migrations, depending on what parameters you pass to the type. By default, with no parameters, the UUID will be stored as :binary , and loaded in :default format. You can control this behaviour by using the :dump and :format options to control what format is used for persistence and in-memory, respectively. The format atoms are the same as you can pass elsewhere, i.e. :raw , :default , :hex , :urn , and :slug . All of them but :raw are printable strings, while :raw is a binary-encoded format. If you wish to use autogenerated UUIDs with Ecto, you have a couple of options: # Generate UUIDv4 primary keys @primary_key { :id , Uniq.UUID , autogenerate : true } schema &quot;foo&quot; do ... end # Generate primary keys using UUIDs of a specific version # NOTE: To autogenerate UUIDs using version 3 or 5, see below @primary_key { :id , Uniq.UUID , version : 1 , autogenerate : true } schema &quot;foo&quot; do ... end # Generate primary keys using, version 3 or 5, which are namespaced # NOTE: Uniq generates 8 bytes of cryptographically strong random data for the name, but # you must provide a custom namespace in which these names are allocated, as the predefined # namespaces are not a good fit for random generated ids. @namespace Uniq.UUID . uuid5 ( :dns , &quot;foo.example.com&quot; , :raw ) @primary_key { :id , Uniq.UUID , version : 5 , namespace : @namespace , autogenerate : true } schema &quot;foo&quot; do ... end # The same rules as above can be used to autogenerate UUIDs for any field, not just primary keys schema &quot;foo&quot; do field :uuidv4 , Uniq.UUID , autogenerate : true field :uuidv1 , Uniq.UUID , version : 1 , autogenerate : true # This field will be dumped to a 36-byte printable string format, and loaded into a 22-byte base64-encoded string field :uuidv3 , Uniq.UUID , version : 3 , namespace : @namespace , format : :slug , dump : :default , autogenerate : true end To use UUIDs for all keys, you can do something like this: # Define your global schema defaults in a module defmodule MyApp.Schema do defmacro __using__ ( _ ) do quote do use Ecto.Schema @primary_key { :id , Uniq.UUID , autogenerate : true } @foreign_key_type Uniq.UUID end end end # Then use that module anywhere that you would use `Ecto.Schema` to apply those defaults defmodule MyApp.Comment do use MyApp.Schema schema &quot;comments&quot; do belongs_to :post , MyApp.Post end end","ref":"using_with_ecto.html","title":"Using With Ecto","type":"extras"},{"doc":"Migration from elixir_uuid is very simple, and you have 2 paths depending on how it is used in your project today. You no longer depend on elixir_uuid directly or indirectly You no longer depend on elixir_uuid directly, but it is still present in your dependency tree In the first scenario, all you need to do is replace any uses of UUID in your project with Uniq.UUID , or simply alias Uniq.UUID in those modules. NOTE: The info/1 and info!/1 functions return a struct by default, so if you use those functions and you aren't planning to add the compatibility shim, you'll want to update those uses. See the function docs for information on the structure. In the second scenario - which also applies if you just want to add the dependency without making any code changes - you must add an override dependency for :elixir_uuid , like so: defp deps do [ { :uniq , &quot;~&gt; x.x&quot; } , { :elixir_uuid , &quot;~&gt; 0.1&quot; , hex : :uniq_compat , override : true } , ] end This replaces the :elixir_uuid dependency with a shim that delegates to :uniq internally. With this in place, elixir_uuid is removed from your dependency tree entirely.","ref":"migration_from_elixir_uuid.html","title":"Migrating From elixir_uuid","type":"extras"},{"doc":"Apache License Version 2.0 , January 2004 http :// www . apache . org / licenses / TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License ; and (b) You must cause any modified files to carry prominent notices stating that You changed the files ; and (c) You must retain, in the Source form of any Derivative Works that You distribute , all copyright , patent , trademark , and attribution notices from the Source form of the Work , excluding those notices that do not pertain to any part of the Derivative Works ; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution , then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file , excluding those notices that do not pertain to any part of the Derivative Works , in at least one of the following places : within a NOTICE text file distributed as part of the Derivative Works ; within the Source form or documentation , if provided along with the Derivative Works ; or , within a display generated by the Derivative Works , if and wherever such third - party notices normally appear . The contents of the NOTICE file are for informational purposes only and do not modify the License . You may add Your own attribution notices within Derivative Works that You distribute , alongside or as an addendum to the NOTICE text from the Work , provided that such additional attribution notices cannot be construed as modifying the License . You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work , attach the following boilerplate notice , with the fields enclosed by brackets &quot;{}&quot; replaced with your own identifying information . ( Don &#39; t include the brackets! ) The text should be enclosed in the appropriate comment syntax for the file format . We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third - party archives . Copyright 2018 Paul Schoenfelder Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http :// www . apache . org / licenses / LICENSE - 2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html","title":"License","type":"extras"}]